<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" />

	<xsl:template match="/pokeydoc">
		<html>
			<head>
				<title><xsl:value-of select="@title" /></title>
				<style>
					h3 { padding-left: 5px; background-color: #000080; color: white; }
					.todo { margin-top: 10px; border: solid black 1px; padding: 5px; background-color: #ffe0c0; }
					.problem_open { margin-top: 10px; border: solid black 1px; padding: 5px; background-color: #ffffd0; }
					.problem_solved { margin-top: 10px; border: solid black 1px; padding: 5px; background-color: #f0fff0; }
					.q { color: #c00000; }
					.a { color: #008000; }
					<xsl:if test="//regs">
						table.regs tr th { padding: 5px 45px 5px 45px; background-color: #000080; color: white; }
						table.regs tr td { padding: 5px 45px 5px 45px; text-align: center; }
						table.regs tr td.not_connected { background-color: #c0c0c0; font-style: italic; }
					</xsl:if>
					<xsl:if test="//pinout">
						td.chip_top { font-size: 12px; border-top: solid black 3px; }
						td.chip_u { font-size: 12px; border: solid black 3px; }
						td.chip_ldesc { font-size: 12px; padding-right: 5px; text-align: right; }
						td.chip_left { font-size: 2px; border-right: solid black 3px; }
						td.chip_lpin { font-size: 12px; border: solid black 1px; border-right: solid black 3px; }
						td.chip_lnum { font-size: 12px; padding-left: 5px; text-align: left; }
						td.chip_rnum { font-size: 12px; padding-right: 5px; text-align: right; }
						td.chip_rpin { font-size: 12px; border: solid black 1px; border-left: solid black 3px; }
						td.chip_right { font-size: 2px; border-left: solid black 3px; }
						td.chip_rdesc { font-size: 12px; padding-left: 5px; text-align: left; }
						td.chip_bottom { font-size: 12px; border-bottom: solid black 3px; }
						table.pins tr th { padding: 0px 5px 0px 5px; background-color: #000080; color: white; }
						table.pins tr td { padding: 0px 5px 0px 5px; }
					</xsl:if>
					<xsl:if test="//keymatrix">
						table.keymatrix tr th { background-color: #f0f0f0; }
						table.keymatrix tr td { text-align: center; }
						table.keymatrix tr td.not_connected { background-color: #e0e0e0; }
						.break { background-color: #ffe0e0; }
						.shift { background-color: #ffffd0; }
						.control { background-color: #ffd0ff; }
						.inverse { background-color: #e0ffff; }
						.help { background-color: #e0e0ff; }
						.function { background-color: #e0ffe0; }
						.button { background-color: #ffe0c0; }
						.regvalue { text-align: center; font-family: monospace; }
						.regspecial { color: red; }
					</xsl:if>
				</style>
			</head>
			<body>
				<h1><xsl:value-of select="@title" /></h1>
				<xsl:apply-templates />
			</body>
		</html>
	</xsl:template>

	<xsl:template match="section">
		<h2><xsl:value-of select="@title" /></h2>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="appendix">
		<hr />
		<h1><xsl:value-of select="@title" /></h1>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="links">
		<h2><xsl:value-of select="@title" /></h2>
		<ul>
			<xsl:for-each select="link">
				<li>
					<xsl:choose>
						<xsl:when test="@file">
							<a href="{@file}.xml"><xsl:apply-templates /></a>
						</xsl:when>
						<xsl:when test="@module">
							<a href="{@module}.xml"><xsl:value-of select="translate(@module, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')" /> module</a>
						</xsl:when>
						<xsl:otherwise>
							<a href="{@href}"><xsl:apply-templates /></a>
						</xsl:otherwise>
					</xsl:choose>
				</li>
			</xsl:for-each>
		</ul>
	</xsl:template>

	<xsl:template match="todo">
		<div class="todo">
			<strong>TODO: </strong>
			<xsl:apply-templates />
		</div>
	</xsl:template>

	<xsl:template match="problem">
		<div>
			<xsl:attribute name="class">
				<xsl:choose>
					<xsl:when test="a">problem_solved</xsl:when>
					<xsl:otherwise>problem_open</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:apply-templates />
		</div>
	</xsl:template>

	<!-- question, answer or comment -->
	<xsl:template match="problem/q|problem/a|problem/c">
		<div class="{name()}">
			<xsl:if test="@by">
				<strong><xsl:value-of select="@by" />: </strong>
			</xsl:if>
			<xsl:apply-templates />
		</div>
	</xsl:template>

	<xsl:template match="modmap">
		<img src="{@src}" usemap="#map" border="0" />
		<map name="map">
			<xsl:for-each select="rect">
				<area shape="rect" coords="{@x + 60},{@y + 57},{@x + 175},{@y + 115}" href="{@module}.xml" />
			</xsl:for-each>
		</map>
	</xsl:template>

	<xsl:template match="regs">
		<table border="1" cellspacing="0" align="center" class="regs">
			<tr>
				<th>Address</th>
				<th>Write</th>
				<th>Read</th>
			</tr>
			<xsl:for-each select="reg">
				<tr>
					<td><code><xsl:value-of select="@addr" /></code></td>
					<xsl:apply-templates select="write" mode="cell" />
					<xsl:apply-templates select="read" mode="cell" />
				</tr>
			</xsl:for-each>
		</table>
		<xsl:apply-templates select="reg/write[@key]|reg/read[@key]" mode="description" />
	</xsl:template>

	<xsl:template match="read|write" mode="cell">
		<td>
			<xsl:choose>
				<xsl:when test="@name">
					<code><a href="#{@key|@ref}"><xsl:value-of select="@name" /></a></code>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="class">not_connected</xsl:attribute>
					<xsl:apply-templates />
				</xsl:otherwise>
			</xsl:choose>
		</td>
	</xsl:template>

	<xsl:template match="read[@key]|write[@key]" mode="description">
		<xsl:variable name="key" select="@key" />
		<a name="{@key}" />
		<h3>
			<xsl:value-of select="@key" />
			<xsl:text> (</xsl:text>
				<xsl:value-of select="name()" />
				<xsl:text> </xsl:text>
				<xsl:value-of select="../@addr" />
				<xsl:for-each select="../../reg[*/@ref=$key]">
					<xsl:text>, </xsl:text>
					<xsl:value-of select="@addr" />
				</xsl:for-each>
			<xsl:text>)</xsl:text>
		</h3>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="bits">
		<xsl:variable name="regkey" select="ancestor::read/@key|ancestor::write/@key" />
		<table border="1" cellspacing="0" cellpadding="10" style="table-layout: fixed">
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<col width="70" />
			<tr>
				<xsl:for-each select="bit">
					<xsl:choose>
						<xsl:when test="@span">
							<td colspan="{@span}" align="center">
								<b>b<xsl:value-of select="@ord" /> - b<xsl:value-of select="1 + @ord - @span" /></b>
							</td>
						</xsl:when>
						<xsl:otherwise>
							<td align="center">
								<b>b<xsl:value-of select="@ord" /></b>
							</td>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</tr>
		</table>
		<table border="0">
			<xsl:for-each select="bit">
				<tr>
					<td width="70" align="center" valign="top">
						<a name="{$regkey}_{@ord}" />
						<b>
							b<xsl:value-of select="@ord" />
							<xsl:if test="@span">
								- b<xsl:value-of select="1 + @ord - @span" />
							</xsl:if>
						</b>
					</td>
					<td><xsl:apply-templates /></td>
				</tr>
			</xsl:for-each>
		</table>
	</xsl:template>

	<xsl:template match="reg[@ref and @bit]">
		<a href="registers.xml#{@ref}_{@bit}">bit <xsl:value-of select="@bit" /> of <xsl:value-of select="@ref" /></a>
	</xsl:template>

	<xsl:template match="reg[@ref and not(@bit)]">
		<a href="registers.xml#{@ref}"><xsl:value-of select="@ref" /></a>
	</xsl:template>

	<xsl:template match="pinout">
		<xsl:variable name="pinout" select="." />
		<table cellspacing="0" cellpadding="0" style="table-layout: fixed" align="left">
			<col width="40px" />
			<col width="16px" />
			<col width="40px" />
			<col width="40px" />
			<col width="40px" />
			<col width="16px" />
			<col width="80px" />
			<tr height="20">
				<td></td>
				<td class="chip_left">&#160;</td>
				<td class="chip_top">&#160;</td>
				<td class="chip_u">&#160;</td>
				<td class="chip_top">&#160;</td>
				<td class="chip_right">&#160;</td>
				<td></td>
			</tr>
			<xsl:for-each select="pin">
				<xsl:if test="position() &lt;= 20">
					<xsl:variable name="p" select="position()" />
					<tr height="16">
						<td class="chip_ldesc"><xsl:value-of select="$pinout/pin[@no = $p]/@name" /></td>
						<td class="chip_lpin">&#160;</td>
						<td class="chip_lnum"><xsl:value-of select="$p" /></td>
						<td></td>
						<td class="chip_rnum"><xsl:value-of select="41 - position()" /></td>
						<td class="chip_rpin">&#160;</td>
						<td class="chip_rdesc"><xsl:value-of select="$pinout/pin[@no = 41 - $p]/@name" /></td>
					</tr>
					<xsl:if test="position() &lt; 20">
						<tr height="8">
							<td></td>
							<td class="chip_left">&#160;</td>
							<td></td>
							<td></td>
							<td></td>
							<td class="chip_right">&#160;</td>
							<td></td>
						</tr>
					</xsl:if>
				</xsl:if>
			</xsl:for-each>
			<tr height="20">
				<td></td>
				<td class="chip_left">&#160;</td>
				<td class="chip_bottom">&#160;</td>
				<td class="chip_bottom">&#160;</td>
				<td class="chip_bottom">&#160;</td>
				<td class="chip_right">&#160;</td>
				<td></td>
			</tr>
		</table>
	</xsl:template>

	<xsl:template match="pindef">
		<table border="1" cellspacing="0" cellpadding="0" class="pins">
			<tr><th>Name</th><th>Direction</th><th>Function</th></tr>
			<xsl:for-each select="pin">
				<tr>
					<td align="center">
						<b><xsl:value-of select="@key" /></b>
					</td>
					<td align="center"><xsl:value-of select="@dir" /></td>
					<td><xsl:apply-templates /></td>
				</tr>
			</xsl:for-each>
		</table>
	</xsl:template>

	<xsl:template match="pin[@ref]">
		<a href="pins.xml"><xsl:value-of select="@ref" /></a>
	</xsl:template>

	<xsl:template match="scans">
		<table border="0" cellspacing="10" callpadding="0" width="100%">
			<col width="33%" />
			<col width="33%" />
			<col width="33%" />
			<tr>
				<xsl:apply-templates select="scan[@no &lt;= 3]" />
			</tr>
			<tr>
				<xsl:apply-templates select="scan[@no &gt; 3 and @no &lt;= 6]" />
			</tr>
			<tr>
				<xsl:apply-templates select="scan[@no &gt; 6]" />
				<td><xsl:apply-templates select="comment/node()" /></td>
				<xsl:apply-templates select="scan[@name]" />
			</tr>
		</table>
	</xsl:template>

	<xsl:template match="scan[@no]">
		<td align="center">
			Page <xsl:value-of select="@no" /> (<xsl:value-of select="34 + @no" />):
			<xsl:value-of select="@label" /><br />
			<a href="p{@no}.png">
				<img src="t{@no}.png" width="255" height="164" />
			</a>
		</td>
	</xsl:template>

	<xsl:template match="scan[@name]">
		<td align="center">
			<xsl:value-of select="@label" /><br />
			<a href="{@name}.png">
				<img src="t{@name}.png" width="255" height="164" />
			</a>
		</td>
	</xsl:template>

	<xsl:template match="scan[@ref]">
		<p>
			See also:
			<a href="p{@ref}.png">
				schema page <xsl:value-of select="@ref" />
				<xsl:if test="@area">
					(<xsl:value-of select="@area" />)
				</xsl:if>
				<xsl:text>: </xsl:text>
				<xsl:apply-templates />
			</a>.
		</p>
	</xsl:template>

	<xsl:template match="table">
		<table border="1" cellspacing="0" cellpadding="5">
			<xsl:apply-templates />
		</table>
	</xsl:template>

	<!-- workaround for Internet Explorer which omits borders for empty table cells -->
	<xsl:template match="td[not(node())]">
		<td>&#160;</td>
	</xsl:template>

	<xsl:template match="keymatrix">
		<h2>Key matrix of Atari <xsl:value-of select="@for" /></h2>
		<table border="1" cellspacing="0" cellpadding="5" class="keymatrix">
			<tr>
				<th>K2 K1 K0 &#8594;</th>
				<xsl:choose>
					<xsl:when test="@for != '5200'">
						<th width="80">0 0 0</th>
						<th width="80">0 0 1</th>
						<th width="80">0 1 0</th>
						<th width="80">0 1 1</th>
						<th width="80">1 0 0</th>
						<th width="80">1 0 1</th>
						<th width="80">1 1 0</th>
						<th width="80">1 1 1</th>
						<th width="80">1 1 1</th>
					</xsl:when>
					<xsl:otherwise>
						<th width="172">0 0 x</th>
						<th width="172">0 1 x</th>
						<th width="172">1 0 x</th>
						<th width="172">1 1 x</th>
						<th width="80">1 1 x</th>
					</xsl:otherwise>
				</xsl:choose>
			</tr>
			<tr>
				<th>K5 K4 K3 &#8595;</th>
				<xsl:choose>
					<xsl:when test="@for != '5200'">
						<th colspan="8">KR1</th>
					</xsl:when>
					<xsl:otherwise>
						<th colspan="4">KR1</th>
					</xsl:otherwise>
				</xsl:choose>
				<th>KR2</th>
			</tr>
			<xsl:choose>
				<xsl:when test="@for != '5200'">
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">0</xsl:with-param><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">0</xsl:with-param><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">0</xsl:with-param><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">0</xsl:with-param><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">1</xsl:with-param><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">1</xsl:with-param><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">1</xsl:with-param><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow"><xsl:with-param name="k5">1</xsl:with-param><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="keyrow5200"><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow5200"><xsl:with-param name="k4">0</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow5200"><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">0</xsl:with-param></xsl:call-template>
					<xsl:call-template name="keyrow5200"><xsl:with-param name="k4">1</xsl:with-param><xsl:with-param name="k3">1</xsl:with-param></xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</table>
		<p>Notes:</p>
		<table border="1" cellspacing="0" cellpadding="5">
			<tr>
				<th>Key</th>
				<th>Comment</th>
			</tr>
			<xsl:for-each select="key[text()]">
				<tr>
					<td align="center" class="{@spec}">
						<xsl:value-of select="@name" />
						<xsl:if test="@name = 'F1'"> - F4</xsl:if>
					</td>
					<td><xsl:apply-templates /></td>
				</tr>
			</xsl:for-each>
		</table>
	</xsl:template>

	<xsl:template name="keyrow">
		<xsl:param name="k5" />
		<xsl:param name="k4" />
		<xsl:param name="k3" />
		<xsl:variable name="k543" select="$k5 * 4 + $k4 * 2 + $k3" />
		<tr>
			<th><xsl:value-of select="$k5" />&#160;<xsl:value-of select="$k4" />&#160;<xsl:value-of select="$k3" /></th>
			<xsl:for-each select="key[position() &lt;= 8]">
				<xsl:variable name="code" select="64 - $k543 * 8 - position()" />
				<xsl:call-template name="keycell"><xsl:with-param name="key" select="../key[@code = $code]" /></xsl:call-template>
			</xsl:for-each>
			<xsl:call-template name="keycell"><xsl:with-param name="key" select="key[@control = $k543]" /></xsl:call-template>
		</tr>
	</xsl:template>

	<xsl:template name="keyrow5200">
		<xsl:param name="k4" />
		<xsl:param name="k3" />
		<xsl:variable name="k543" select="$k4 * 2 + $k3" />
		<tr>
			<th>x&#160;<xsl:value-of select="$k4" />&#160;<xsl:value-of select="$k3" /></th>
			<xsl:for-each select="key[position() &lt;= 4]">
				<xsl:variable name="code" select="65 - $k543 * 8 - 2 * position()" />
				<xsl:call-template name="keycell"><xsl:with-param name="key" select="../key[@code = $code]" /></xsl:call-template>
			</xsl:for-each>
			<xsl:call-template name="keycell"><xsl:with-param name="key" select="key[@control = $k543]" /></xsl:call-template>
		</tr>
	</xsl:template>

	<xsl:template name="keycell">
		<xsl:param name="key" />
		<xsl:choose>
			<xsl:when test="$key">
				<td>
					<xsl:if test="$key/@spec"><xsl:attribute name="class"><xsl:value-of select="$key/@spec" /></xsl:attribute></xsl:if>
					<xsl:if test="$key/@code">
						<xsl:attribute name="title">
							<xsl:text>key: </xsl:text>
							<xsl:value-of select="$key/@name" />
							<xsl:text>  decimal code: </xsl:text>
							<xsl:value-of select="$key/@code" />
							<xsl:text>  hex code: </xsl:text>
							<xsl:call-template name="hexbyte">
								<xsl:with-param name="num" select="$key/@code" />
							</xsl:call-template>
						</xsl:attribute>
					</xsl:if>
					<xsl:value-of select="$key/@name" />
				</td>
			</xsl:when>
			<xsl:otherwise><td class="not_connected">&#160;</td></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="hexbyte">
		<xsl:param name="num" />
		<xsl:variable name="low" select="$num mod 16" />
		<xsl:call-template name="hexdigit">
			<xsl:with-param name="digit" select="($num - $low) div 16" />
		</xsl:call-template>
		<xsl:call-template name="hexdigit">
			<xsl:with-param name="digit" select="$low" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="hexdigit">
		<xsl:param name="digit" />
		<xsl:value-of select="substring('0123456789ABCDEF', 1 + $digit, 1)" />
	</xsl:template>

	<xsl:template match="shiftreg">
		<table border="1" cellspacing="0" cellpadding="5">
			<tr>
				<th colspan="2"><xsl:value-of select="@name" /></th>
			</tr>
			<tr>
				<th>Register value</th>
				<th>Sequence number or comment</th>
			</tr>
			<xsl:apply-templates />
		</table>
	</xsl:template>

	<xsl:template match="state">
		<tr>
			<td class="regvalue"><xsl:value-of select="@value" /></td>
			<td><xsl:apply-templates /></td>
		</tr>
	</xsl:template>

	<xsl:template match="special">
		<tr class="regspecial">
			<td class="regvalue"><xsl:value-of select="@value" /></td>
			<td><xsl:apply-templates /></td>
		</tr>
	</xsl:template>

	<xsl:template match="a[@mail]">
		<a href="mailto:{@mail}"><xsl:apply-templates /></a>
	</xsl:template>

	<xsl:template match="a[@href]|br|hr|img|map|area|p|pre|code|strong|tr|td|th|ul|li">
		<xsl:copy>
			<xsl:copy-of select="@*" />
			<xsl:apply-templates />
		</xsl:copy>
	</xsl:template>

	<xsl:template match="*">
		<xsl:message terminate="yes">Unknown tag: <xsl:value-of select="name()" /></xsl:message>
	</xsl:template>

</xsl:stylesheet>
